using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Xml;
using System.Net;
using System.IO;
using System.Text;
using System.Xml.XPath;



public class RssFeedAsyncReader
{
    private WebRequest _req;
    private string _rssData;
    private string _blogUrl;

    public RssFeedAsyncReader(string blog)
    {
        _blogUrl = blog;
    }

    public IAsyncResult BeginRead(object sender, EventArgs e, AsyncCallback cb, object state)
    {
        // Prepare to make a Web request
        _req = WebRequest.Create(_blogUrl);

        // Begins the operation and return an object that implements IAsyncResult
        IAsyncResult obj = _req.BeginGetResponse(cb, state);
        return obj;
    }

    public void EndRead(IAsyncResult ar)
    {
        string text;
        using (WebResponse response = _req.EndGetResponse(ar))
        {
            using (StreamReader reader = new StreamReader(response.GetResponseStream()))
            {
                text = reader.ReadToEnd();
            }

            // Process the RSS data
            _rssData = ProcessFeed(text);
        }
    }

    string ProcessFeed(string feed)
    {
        StringBuilder sb = new StringBuilder();

        XmlDocument doc = new XmlDocument();
        doc.LoadXml(feed);

        XPathNavigator nav = doc.CreateNavigator();
        XPathNodeIterator iterator;

        // Title and description
        iterator = nav.Select("/rss/channel/title");
        iterator.MoveNext();
        sb.AppendFormat("<h3>{0}</h3>", iterator.Current.Value);
        iterator = nav.Select("/rss/channel/description");
        iterator.MoveNext();
        sb.AppendFormat("<i>{0}</i>", iterator.Current.Value);

        XmlNodeList items = doc.SelectNodes("/rss/channel/item");
        sb.Append("<ul>");
        foreach (XmlNode n in items)
        {
            XmlNode title = n.SelectSingleNode("title");
            XmlNode link = n.SelectSingleNode("link");
            XmlNode date = n.SelectSingleNode("pubDate");
            sb.AppendFormat("<li><a href=\"{0}\">{1}</a> on {2}</li>", link.InnerText, title.InnerText, date.InnerText);
        }
        sb.Append("</ul>");

        return sb.ToString();
    }

    public string GetRssData()
    {
        return _rssData;
    }
}
