<%@ WebHandler Language="C#" Class="ChartHandler" %>

using System;
using System.Web;
using System.Runtime.Remoting;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Configuration;
using System.Data;
using ProAspNet20.Advanced.CS.Components;


public class ChartHandler : IHttpHandler 
{
   
    public void ProcessRequest (HttpContext context) 
    {
        // Process arguments
        string type = "bar";
        string dataProvider = String.Empty;
        object arg1 = context.Request["Type"];
        object arg2 = context.Request["Data"];
        if (arg1 != null)
            type = (string)arg1;
        if (arg2 == null)
        {
            context.Response.End();
            return;
        }
        else
            dataProvider = (string)arg2;
        
        // Retrieve the data provider
        string info = ConfigurationManager.AppSettings[dataProvider];
        string[] typeInfo = info.Split(',');
        string cls = typeInfo[0];
        string asm = typeInfo[1];
        ObjectHandle handle = Activator.CreateInstance(asm, cls);
        ChartDataProvider prov = (ChartDataProvider)handle.Unwrap();

        // Draw the chart
        Bitmap bmp;
        Graphics g;
        if (type == "bar")
        {
            bmp = new Bitmap(700, 200);
            g = Graphics.FromImage(bmp);
            DrawBarChart(g, prov);
        }
        else
        {
            bmp = new Bitmap(300, 300);
            g = Graphics.FromImage(bmp);
            DrawPieChart(g, prov);
        }

        // Send bits
        context.Response.ContentType = "image/png";
        MemoryStream ms = new MemoryStream();
        bmp.Save(ms, ImageFormat.Png);
        ms.WriteTo(context.Response.OutputStream);
        bmp.Dispose();
        g.Dispose();
    }
 
    public bool IsReusable 
    {
        get { return true; }
    }

    
    #region Helpers
    private void DrawBarChart(Graphics g, ChartDataProvider prov)
    {
        // Get the data to represent
        DataTable dt = prov.GetData(); 

        // Get the total of sales
        string maxValueExpr = String.Format("max({0})", prov.DataField);
        float maxValue = Convert.ToSingle(dt.Compute(maxValueExpr, ""));

        // Prepare for rendering
        int m_Width = 700;
        int m_MaxBarHeight = 150;
        int m_MinYPos = 20;
        int m_TextHeight = 15;
        int m_ShadowDepth = 7;
        int barWidth = m_Width / dt.Rows.Count;

        // Clear the background
        g.Clear(Color.White);

        // Draw the bars
        for (int j = m_ShadowDepth; j > 0; j--)
        {
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                float value = Convert.ToSingle(dt.Rows[i][prov.DataField]) / maxValue * m_MaxBarHeight;
                int barBottom = m_MinYPos + m_MaxBarHeight;
                int barHeight = (int)value;
                int yBarTop = barBottom - barHeight;
                int yCaption = barBottom - barHeight - m_TextHeight - m_ShadowDepth;

                int x = i * barWidth;
                g.DrawString(String.Format("{0:c}", dt.Rows[i][prov.DataField]),
                    new Font("Tahoma", 8),
                    new SolidBrush(Color.Blue), x, yCaption);


                // Calculate and fill the bar		
                Rectangle bar = new Rectangle(x, yBarTop, barWidth - 10, barHeight);
                Rectangle rcShadow = new Rectangle(bar.Location, bar.Size);
                rcShadow.Offset(j, -j);
                g.FillRectangle(new HatchBrush(HatchStyle.Percent50, Color.Orange), rcShadow);
                g.FillRectangle(new LinearGradientBrush(bar, Color.Orange, Color.Yellow, LinearGradientMode.Horizontal), bar);

                Font fnt = new Font("Tahoma", 8, FontStyle.Bold);
                g.DrawString(dt.Rows[i][prov.DataLabelField].ToString(), fnt,
                    new SolidBrush(Color.Blue), x, barBottom + m_TextHeight);
            }
        }
    }

    private void DrawPieChart(Graphics g, ChartDataProvider prov)
    {
        // Get the data to represent
        DataTable dt = prov.GetData();
  
    	// Get the total of sales
        string sumValueExpr = String.Format("sum({0})", prov.DataField);
	    float fTotal = Convert.ToSingle(dt.Compute(sumValueExpr, ""));

        // Prepare for rendering
	    float angle = 0.0F;
	    Rectangle area = new Rectangle(0, 0, 280, 280);
	    Color[] colors = {Color.Chocolate, Color.YellowGreen, Color.DarkGoldenrod, Color.DarkKhaki, Color.Orange, Color.PaleGoldenrod, Color.Peru, Color.Tan, Color.Khaki, Color.MediumSpringGreen};
	    Color c;
	    Point[] centers = new Point[dt.Rows.Count];

        // Clear the background
       	g.SmoothingMode = SmoothingMode.AntiAlias;
        g.Clear(Color.White);

	    // Draw the wedges
	    for(int j=15; j>0; j--)
	    {
		    for(int i=0; i<dt.Rows.Count; i++)
		    {
			    // angle for the value
			    float sweep = Convert.ToSingle(dt.Rows[i][prov.DataField])/fTotal*360;

			    // color 
			    c = colors[i];
    			
			    // shadow
			    Rectangle shadowArea = new Rectangle(area.Location, area.Size);
			    shadowArea.Offset(j, j);
			    g.FillPie(new HatchBrush(HatchStyle.Percent50, c), shadowArea, angle, sweep);
    			
			    // wedge
			    g.FillPie(new SolidBrush(c), area, angle, sweep);
			    g.DrawEllipse(new Pen(Color.DarkGray, 1), area);
    			
			    // center of the wedge to use later to draw labels
			    Point center = GetPoint(angle + sweep/2, area.Width, area.Height);
                center.X = (int) ((area.Right - area.Left) / 2 + center.X) / 2;
                center.Y = (int) ((area.Bottom - area.Top) / 2 + center.Y) / 2;
                centers[i] = center;
    			
			    angle += sweep;
		    }
	    }
	
	    for(int i=0; i<dt.Rows.Count; i++)
	    {
		    g.DrawString(dt.Rows[i]["Employee"].ToString(), 
			    new Font("Tahoma", 8), 
			    new SolidBrush(Color.Black), 
			    centers[i].X, centers[i].Y);
	    }
    }

    private Point GetPoint(float angle, int width, int height)
    {
        int radius = width / 2;
        Point topCenter = new Point(radius, 0);
        double rad = Math.PI * 2 * angle / 360;

        Point pt = new Point();
        pt.X = (int)(radius * Math.Cos(rad)) + width / 2;
        pt.Y = (int)(radius * Math.Sin(rad)) + height / 2;

        //pt.X = (int) (Math.Cos(rad) * topCenter.X - Math.Sin(rad) * topCenter.Y) + width / 2;
        //pt.Y = (int) (Math.Sin(rad) * topCenter.X + Math.Cos(rad) * topCenter.Y) + height / 2;

        return pt;
    }
    
    #endregion
}