using System;
using System.ComponentModel;
using System.Collections;
using System.Drawing;
using System.Web.UI;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;


namespace ProAspNet20.Advanced.CS.Components
{
    public sealed class AutoRefreshWebPart : WebPart, ICallbackEventHandler
    {
        #region Private Members
		private int _refreshRate;
		private Random _rnd;
        #endregion


        #region ICallbackEventHandler
        private string _results;
        void ICallbackEventHandler.RaiseCallbackEvent(string argument)
        {
            _rnd = new Random();
            int r = _rnd.Next(0, 255);
            int g = _rnd.Next(0, 255);
            int b = _rnd.Next(0, 255);

            _results = String.Format("#{0:x}{1:x}{2:x}", r, g, b);
            return;
        }
        string ICallbackEventHandler.GetCallbackResult()
        {
            return _results;
        }
        #endregion


        #region Overrides
        public override string Title
		{
			get
			{
                if (String.IsNullOrEmpty(base.Title))
				    return "AutoRefresh";
                return base.Title;
			}
			set
			{
				base.Title = value;
			}
		}
        public override string Subtitle
        {
            get
            {
                if (String.IsNullOrEmpty(base.Subtitle))
                    return "AutoRefresh";
                return base.Subtitle;
            }
        }
		#endregion


		#region Properties
		// ************************************************************************
		// Gets and sets the refresh rate (in seconds) for the stock list
		[Personalizable]
		[WebBrowsable]
		public int RefreshRate 
		{
			get { return _refreshRate; }
			set { _refreshRate = value; }
		}
		#endregion


		// ************************************************************************
		// Render out the Web part
		protected override void RenderContents (HtmlTextWriter writer)
        {
			CreateOutput(writer);
		}
 
		#region Helper Methods
		// ************************************************************************
		// Build a table with current stock values
        private void CreateOutput(HtmlTextWriter writer)
		{
            Table t = new Table();
            t.ID = "Table1";
            t.CopyBaseAttributes(this);
            t.Width = Unit.Percentage(100);
            t.BackColor = Color.Blue;

            TableRow row = new TableRow();
            t.Rows.Add(row);
            TableCell c1 = new TableCell();
            c1.HorizontalAlign = HorizontalAlign.Left;
            c1.Text = "&nbsp;";
            row.Cells.Add(c1);

            // Add some script 
			if (RefreshRate > 0)
			{
				if (!Page.ClientScript.IsStartupScriptRegistered(this.GetType(), "AutoRefresh"))
				{
					string js2 = GetRefreshScript();
                    Page.ClientScript.RegisterStartupScript(this.GetType(), "AutoRefresh", js2, true);
				}
			}

            // Render out
            t.RenderControl(writer);
		}

        private string GetRefreshScript()
        {
            StringBuilder sb = new StringBuilder("\r\nfunction AutoRefresh() {\r\n");
            sb.AppendLine(Page.ClientScript.GetCallbackEventReference(this, "null", "UpdateColor", "null"));
            sb.AppendFormat("  window.setTimeout(\"AutoRefresh()\", {0});", RefreshRate * 1000);
			 sb.AppendLine("\r\n}");
            sb.AppendLine(GetUpdateColor());
            sb.AppendLine("AutoRefresh();");

            return sb.ToString();
        }

        private string GetUpdateColor()
        {
            StringBuilder sb = new StringBuilder("function UpdateColor(response, context) {\r\n");
        //    sb.AppendLine("  alert(response); ");
            sb.AppendLine("  var clr = response;");
            sb.AppendLine("  var tbl = document.getElementById(\"Table1\");");
            sb.AppendFormat("  tbl.style.backgroundColor = clr;");
            sb.AppendLine("}");
            return sb.ToString();
        }
		#endregion
	}
}

