using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Collections;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;


namespace ProAspNet20.Advanced.CS.Components
{
    [ToolboxData("<{0}:BarChart runat=\"server\"></{0}:BarChart>")]
    [DefaultProperty("Title")]
    [DefaultEvent("BarChartCreated")]
    public class BarChart : System.Web.UI.WebControls.CompositeDataBoundControl
    {
        public BarChart()
        {
        }

        #region PRIVATE MEMBERS
        private ITemplate _titleTemplate = null;
        private TitleTemplateContainer _titleTemplateContainer; 
        private BarChartItemCollection _items;
        private TableItemStyle _titleStyle;
        private TableItemStyle _subtitleStyle;
        private TableItemStyle _labelStyle;
        private TableItemStyle _valueStyle;
        private TableItemStyle _barStyle;
        #endregion

        #region CONSTANTS
        //private const string ViewStateItemCount = "BarChartItemCount";
        private const int ColumnsCount = 2;
        private const string NoDataSourceSelected = "No data source selected.";
        #endregion

        #region EVENTS
        public event EventHandler<BarChartItemEventArgs> BarChartCreated;
        public event EventHandler<BarChartItemEventArgs> BarChartDataBound;
        protected virtual void OnBarChartCreated(BarChartItemEventArgs e)
        {
            if (BarChartCreated != null)
                BarChartCreated(this, e);
        }
        protected virtual void OnBarChartDataBound(BarChartItemEventArgs e)
        {
            if (BarChartDataBound != null)
                BarChartDataBound(this, e);
        }        
        #endregion

        #region PROPERTIES

        #region Title
        // *******************************************************
        // Gets and sets the title of the chart
        public string Title
        {
            get
            {
                object o = ViewState["Title"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["Title"] = value; }
        }
        #endregion

        #region SubTitle
        // *******************************************************
        // Gets and sets the subtitle of the chart
        public string SubTitle
        {
            get
            {
                object o = ViewState["SubTitle"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["SubTitle"] = value; }
        }
        #endregion

        #region Maximum
        // *******************************************************
        // Gets and sets the maximum value to represent 
        public float Maximum
        {
            get
            {
                object o = ViewState["Maximum"];
                if (o == null)
                    return 100;
                return (float) o;
            }
            set { ViewState["Maximum"] = value; }
        }
        #endregion

        #region DataTextField
        // *******************************************************
        // Gets and sets the data text field
        public string DataTextField
        {
            get
            {
                object o = ViewState["DataTextField"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["DataTextField"] = value; }
        }
        #endregion

        #region DataTextFormatString
        // *******************************************************
        // Gets and sets the format for the data text field
        public string DataTextFormatString
        {
            get
            {
                object o = ViewState["DataTextFormatString"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["DataTextFormatString"] = value; }
        }
        #endregion

        #region DataValueField
        // *******************************************************
        // Gets and sets the data text field
        public string DataValueField
        {
            get
            {
                object o = ViewState["DataValueField"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["DataValueField"] = value; }
        }
        #endregion

        #region DataValueFormatString
        // *******************************************************
        // Gets and sets the format for the value field
        public string DataValueFormatString
        {
            get
            {
                object o = ViewState["DataValueFormatString"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["DataValueFormatString"] = value; }
        }
        #endregion

        #region Items
        // *******************************************************************************************
        // PROPERTY: Items
        // Gets a collection object that represents the individual visual items in the control
        [Browsable(false)]
        public virtual BarChartItemCollection Items
        {
            get
            {
                if (_items == null)
                    _items = new BarChartItemCollection();

                return _items;
            }
        }
    #endregion

        #endregion

        #region TEMPLATES
        // *******************************************************
        // Sets the template for the chart header
        [Browsable(false)]
        [TemplateContainer(typeof(TitleTemplateContainer))]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public ITemplate TitleTemplate
        {
            get { return _titleTemplate; }
            set { _titleTemplate = value; }
        }
        #endregion

        #region STYLES

        #region PreparingControlForRendering
        // *******************************************************************************************
        // PROPERTY: PreparingControlForRendering 
        // Apply styles to the control hierarchy immediately before rendering
        protected virtual void PreparingControlForRendering()
        {
            // Make sure there are controls to work with
            if (Controls.Count != 1)
                return;

            // Apply the table style
            Table t = (Table)Controls[0];
            t.CopyBaseAttributes(this);
            if (ControlStyleCreated)
                t.ApplyStyle(ControlStyle);

            // Style the title
            //t.Rows[0].Cells[0].ApplyStyle(TitleStyle);
            t.Rows[0].Cells[0].MergeStyle(TitleStyle);

            // Style the subtitle
            //t.Rows[1].Cells[0].ApplyStyle(SubTitleStyle);
            t.Rows[1].Cells[0].MergeStyle(SubTitleStyle);

            // Apply style to the labels that render team names
            for (int i = 2; i < Items.Count + 2; i++)
            {
                // Style team labels
                t.Rows[i].Cells[0].ApplyStyle(LabelStyle);
                t.Rows[i].Cells[1].ApplyStyle(ValueStyle);
                Label lblGraph = (Label)t.Rows[i].Cells[1].Controls[0];
                lblGraph.ControlStyle.MergeWith(BarStyle);
            }
        }
        #endregion

        #region PROPERTY: TitleStyle
        // *******************************************************************************************
        // PROPERTY: TitleStyle 
        // Specifies the style for the header
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [NotifyParentProperty(true)]
        [Description("Specifies the style for the header")]
        public virtual TableItemStyle TitleStyle
        {
            get
            {
                if (_titleStyle == null)
                {
                    _titleStyle = new TableItemStyle();
                    // Can initialize the header HERE 
                }

                if (IsTrackingViewState)
                    ((IStateManager)_titleStyle).TrackViewState();

                return _titleStyle;
            }
        }
        #endregion

        #region PROPERTY: SubTitleStyle
        // *******************************************************************************************
        // PROPERTY: SubTitleStyle 
        // Specifies the style for the subtitle
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [NotifyParentProperty(true)]
        [Description("Specifies the style for the subtitle")]
        public virtual TableItemStyle SubTitleStyle
        {
            get
            {
                if (_subtitleStyle == null)
                {
                    _subtitleStyle = new TableItemStyle();
                    // Can initialize the header HERE 
                }

                if (IsTrackingViewState)
                    ((IStateManager)_subtitleStyle).TrackViewState();

                return _subtitleStyle;
            }
        }
        #endregion

        #region PROPERTY: LabelStyle
        // *******************************************************************************************
        // PROPERTY: LabelStyle 
        // Specifies the style for labels  
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [NotifyParentProperty(true)]
        [Description("Specifies the style for the bar labels")]
        public virtual TableItemStyle LabelStyle
        {
            get
            {
                if (_labelStyle == null)
                {
                    _labelStyle = new TableItemStyle();
                    // Can initialize the style HERE 
                }

                if (IsTrackingViewState)
                    ((IStateManager)_labelStyle).TrackViewState();

                return _labelStyle;
            }
        }
        #endregion

        #region PROPERTY: ValueStyle
        // *******************************************************************************************
        // PROPERTY: ValueStyle 
        // Specifies the style for values  
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [NotifyParentProperty(true)]
        [Description("Specifies the style for the bar values")]
        public virtual TableItemStyle ValueStyle
        {
            get
            {
                if (_valueStyle == null)
                {
                    _valueStyle = new TableItemStyle();
                    // Can initialize the style HERE 
                }

                if (IsTrackingViewState)
                    ((IStateManager)_valueStyle).TrackViewState();

                return _valueStyle;
            }
        }
        #endregion

        #region PROPERTY: BarStyle
        // *******************************************************************************************
        // PROPERTY: BarStyle 
        // Specifies the style for bars  
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [NotifyParentProperty(true)]
        [Description("Specifies the style for the bars")]
        public virtual TableItemStyle BarStyle
        {
            get
            {
                if (_barStyle == null)
                {
                    _barStyle = new TableItemStyle();
                    // Can initialize the style HERE 
                }

                if (IsTrackingViewState)
                    ((IStateManager)_barStyle).TrackViewState();

                return _barStyle;
            }
        }
        #endregion

        #endregion

        #region METHODS

        #region CreateChildControls
        protected override int CreateChildControls(IEnumerable dataSource, bool dataBinding)
        {
            return CreateControlHierarchy(dataSource, dataBinding);
        }
        #endregion


        #region CreateControlHierarchy
        // *******************************************************************************************
        // METHOD: CreateControlHierarchy
        // Create the graph of controls to be rendered
        protected virtual int CreateControlHierarchy(IEnumerable dataSource, bool dataBinding)
        {
            // Get the data to display (either from data source or viewstate)
            if (dataSource == null)
            {
                RenderEmptyControl();
                return 0;
            }

            // Start building the hierarchy of controls
            Table t = new Table();

            // Append the table to the control's tree (ESSENTIAL here)
            // (In general, when building composite control you MUST add them to the hierarchy
            // as soon as possible, immediately after creation)
            Controls.Add(t);

            // Add the header row with the caption 
            //if (ShowTitle)
            CreateTitle(t);

            // Add the subtitle row
            //if (ShowSubTitle)
            CreateSubTitle(t);

            // Build the body of the control's output
            int totalItems = CreateAllItems(t, dataSource, dataBinding);

            // if(ShowFooter)
            //CreateFooter(t, dataSource, dataBinding);

            return totalItems;
        }
        #endregion


        #region RenderEmptyControl
        // *******************************************************************************************
        // METHOD: RenderEmptyControl
        // Used when the control has no UI to show
        protected virtual void RenderEmptyControl()
        {
            LiteralControl lbl = new LiteralControl(BarChart.NoDataSourceSelected);
            Controls.Add(lbl);
        }
        #endregion


        #region CreateTitle
        // *******************************************************************************************
        // METHOD: CreateTitle 
        // Create the title row for the control with first-level caption  
        private void CreateTitle(Table t)
        {
            // Create the table row
            BarChartItem item = new BarChartItem(BarChartItemType.Title);
            t.Rows.Add(item);

            // Add the title cell
            TableCell cell = new TableCell();
            cell.ColumnSpan = BarChart.ColumnsCount;
            item.Cells.Add(cell);

            // Decide between plain string and template
            if (TitleTemplate != null)
            {
                _titleTemplateContainer = new TitleTemplateContainer(this);
                TitleTemplate.InstantiateIn(_titleTemplateContainer);
                cell.Controls.Add(_titleTemplateContainer);
            }
            else
                cell.Text = Title;

            // Must call DataBind to enable #-expression on templates
            item.DataBind();
        }
        #endregion


        #region CreateSubTitle
        // *******************************************************************************************
        // METHOD: CreateSubTitle 
        // Create the sub-title row for the control with a second-level caption  
        private void CreateSubTitle(Table t)
        {
            // Create the table row
            BarChartItem item = new BarChartItem(BarChartItemType.SubTitle);
            t.Rows.Add(item);

            // Decide HERE between templated and default header (default header here)
            TableCell cell = new TableCell();
            cell.ColumnSpan = BarChart.ColumnsCount;

            //
            // Fire HERE the ItemCreated event
            //  	

            item.Cells.Add(cell);
            cell.Text = SubTitle;
        }
        #endregion


        #region CreateAllItems
        // *******************************************************************************************
        // METHOD: CreateAllItems
        // Create the table rows that render ongoing matches
        private int CreateAllItems(Table t, IEnumerable data, bool useDataSource)
        {
            // Count how many items we add 
            int itemCount = 0;

            // Clears the Items collection (creates it, if null)
            Items.Clear();

            // Scroll data items and create table items
            foreach (object o in data)
            {
                // Create the match item object 
                BarChartItemType itemType = BarChartItemType.Item;
                BarChartItem item = CreateBarChartItem(t, itemType, o, useDataSource);

                // Add the newly created object to the Items collection
                // (Using _items--created at the top of the proc--saves you GET accessors) 
                _items.Add(item);

                // Increase the counter
                itemCount++;
            }

            // Return how many items we have into the viewstate (for postbacks)
            return itemCount;
        }
        #endregion

        #endregion


        #region Rendering
		// *******************************************************************************************
		// METHOD: Render
		// Overridden to style controls 
		protected override void Render(HtmlTextWriter output)
		{
			// Style controls before rendering
			PreparingControlForRendering();

			// Avoid the surrounding <span> tag
			base.RenderContents(output);
		}
		#endregion


        #region Helpers
        
        #region CreateBarChartItem
        // *******************************************************************************************
        // METHOD: CreateBarChartItem
        // Create the individual table row representing a single match 
        private BarChartItem CreateBarChartItem(Table t, BarChartItemType itemType, object dataItem, bool useDataSource)
        {
            // Create a new row for the outermost table
            BarChartItem item = new BarChartItem(itemType);

            // Create cells for label and value 
            TableCell labelCell = CreateLabelCell(item);
            TableCell valueCell = CreateValueCell(item);

            // Fire HERE a ItemCreated event 
            BarChartItemEventArgs argsCreated = new BarChartItemEventArgs(item);
            OnBarChartCreated(argsCreated);


            // Add the row to the table  -- ESSENTIAL that you do this HERE
            t.Rows.Add(item);

            // Handle the data object binding
            if (useDataSource)
            {
                // Get the data source object
                item.DataItem = dataItem;

                // Data bind the team labels
                BindLabelCell(labelCell, dataItem);
                BindValueCell(valueCell, dataItem);

                // Fire HERE a ItemDataBound event 
                BarChartItemEventArgs argsDataBound = new BarChartItemEventArgs(item);
                OnBarChartDataBound(argsDataBound);
            }

            // Return the fully configured row item
            return item;
        }
        #endregion

        #region CreateLabelCell
        // *******************************************************************************************
        // METHOD: CreateLabelCell
        // Build the cell to hold the bar label
        private TableCell CreateLabelCell(BarChartItem item)
        {
            // Create the cell
            TableCell cell = new TableCell();

            //
            // Fire HERE a CellItemCreated event (if needed)
            // 

            // Add the cell and return
            item.Cells.Add(cell);
            return cell;
        }
        #endregion

        #region CreateValueCell
        // *******************************************************************************************
        // METHOD: CreateValueCell
        // Build the cell to hold the bar value
        private TableCell CreateValueCell(BarChartItem item)
        {
            // Create the cell
            TableCell cell = new TableCell();

            //
            // Fire HERE a CellItemCreated event (if needed)
            // 

            // Add the cell 
            item.Cells.Add(cell);

            // Add the internal labels
            Label lblGraph = new Label();
            Label lblText = new Label();

            cell.Controls.Add(lblGraph);
            cell.Controls.Add(new LiteralControl("<br>"));
            cell.Controls.Add(lblText);

            return cell;
        }
        #endregion

        #region BindLabelCell
        // *******************************************************************************************
        // METHOD: BindLabelCell
        // Fill the cell with the label name
        private void BindLabelCell(TableCell cell, object dataItem)
        {
            // Called only if we're binding
            if (!String.IsNullOrEmpty(DataTextField))
                cell.Text = DataBinder.GetPropertyValue(dataItem, DataTextField, DataTextFormatString);
            else
                cell.Text = "Label";
        }
        #endregion

        #region BindValueCell
        // *******************************************************************************************
        // METHOD: BindValueCell
        // Fill the cell with the label name
        private void BindValueCell(TableCell cell, object dataItem)
        {
            // Bind the label for the graph
            Label lblGraph = (Label)cell.Controls[0];
            object o = null;
            if (!String.IsNullOrEmpty(DataValueField))
                o = DataBinder.GetPropertyValue(dataItem, DataValueField);
            else
            {
                o = 70;
            }
            float val = Convert.ToSingle(o);
            float valueToRepresent = 100 * val / Maximum;
            lblGraph.Width = Unit.Percentage(valueToRepresent);

            // Bind the label for the text
            Label lblText = (Label)cell.Controls[2];
            if (!String.IsNullOrEmpty(DataValueField))
                lblText.Text = DataBinder.GetPropertyValue(dataItem, DataValueField, DataValueFormatString);
            else
                lblText.Text = o.ToString();
        }
        #endregion

        #endregion

    }
}
