using System;
using System.Data;
using System.Collections.ObjectModel;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;


namespace ProAspNet20.Advanced.CS.Components
{
    #region ENUM::BarChartItemType 
    public enum BarChartItemType
    {
        Title,
        SubTitle,
        Item,
        Footer
    }
    #endregion


    #region CLASS::BarChartItem 
	[ToolboxItem(false)]
    public class BarChartItem : TableRow, INamingContainer
    {
	    #region Private members
	    // ****************************************************************************************
	    // Private members
	    private object _dataItem;
	    private BarChartItemType _itemType;

	    #endregion


	    #region Constructor(s)
	    // ****************************************************************************************
	    // Ctor
	    public BarChartItem(BarChartItemType itemType)
	    {
		    _itemType = itemType;
	    }

	    #endregion


	    #region Properties: DataItem, ItemType
	    // ****************************************************************************************
	    // PROPERTY: DataItem
	    // Gets or sets the data item represented associated with the current bar chart item
	    public object DataItem
	    {
		    get {return _dataItem;}
		    set {_dataItem = value;}
	    }

	    // ****************************************************************************************
	    // PROPERTY: ItemType
	    // Gets the type of the current bar chart item 
	    public BarChartItemType ItemType
	    {
		    get {return _itemType;}
	    }
	    #endregion

    }
    #endregion


    #region CLASS::BarChartItemCollection
    public class BarChartItemCollection : Collection<BarChartItem>
    {
    }
    #endregion


    #region CLASS::BarChartItemEventArgs
    public class BarChartItemEventArgs : EventArgs
    {
        private BarChartItem _item;
        public BarChartItemEventArgs(BarChartItem item)
        {
            _item = item;
        }
 
        // Properties
        public BarChartItem Item 
        {
            get { return _item; }
        }
    }
    #endregion



    #region CLASS::TitleTemplateContainer
    public class TitleTemplateContainer : WebControl, INamingContainer
    {
        private BarChart _parent;
        public TitleTemplateContainer(BarChart parent)
        {
            _parent = parent;
        }
        public string Title
        {
            get { return _parent.Title; }
        }
        public string Subtitle
        {
            get { return _parent.SubTitle; }
        }
        public BarChart BarChart
        {
            get { return _parent; }
        }
    }
    #endregion
}
