using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Data;
using System.ComponentModel;
using System.Data.SqlClient;


// Create an employee information interface that define the communication mechanism
public interface IEmployeeInfo
{
	// Use this property to get the employeed ID currently defined in the control
	int EmployeeID { get; set;} 
}


namespace ProAspNet20.Advanced.CS.Components
{
    public class EmployeesWebPart : WebPart, IEmployeeInfo
    {
        // ************************************************************************
        // Private Members
        private int _empID = 1;
        private DataTable _data = null;
        private TextBox txt;
        private Button btn;

        // ************************************************************************
        // PROPERTY IEmployeeInfo.EmployeeID 
		[Personalizable(true)]
		[WebBrowsable(true)]
		public int EmployeeID
		{
            get { return _empID; }
			set { _empID = value; }
		}

		// ************************************************************************
		// PROPERTY EmployeeID configurable 
		/*
		[Personalizable(true), WebBrowsable(true)]
		public int EmployeeID
		{
			get { return _empID; }
			set { _empID = value; }
		}
		*/


		// Define the callback method that returns the connection data
		[ConnectionProvider("EmployeeIDProvider", "EmployeeIDProvider")]
		public IEmployeeInfo GetEmployeeInfo()
		{
			return this;
		}

        // ************************************************************************
        // Create helper controls
        protected override void CreateChildControls()
        {
			txt = new TextBox();
			txt.Width = Unit.Pixel(30);
			txt.Text = EmployeeID.ToString();
			Controls.Add(txt);

			btn = new Button();
			btn.Text = "Suchen";
			btn.Click += new EventHandler(OnFindEmployee);
			Controls.Add(btn); 
        }

        // ************************************************************************
        // Renders the UI of the Web part
        protected override void RenderContents (HtmlTextWriter writer)
        {
            if (_empID < 1)
            {
                writer.Write ("<span style='color:gray'>Gegenwrtig ist kein Angestellter ausgewhlt.</span>");
                return;
            }

            // Grab info and generate the UI
            FindEmployeeInfo();
            BuildUI (writer);
        }

        // ************************************************************************
        // Retrieves employee information
        private void FindEmployeeInfo()
        {
            if (_empID <1)
                return;
            SqlDataAdapter adapter;
            adapter = new SqlDataAdapter(
                "SELECT * FROM employees WHERE employeeid=" + _empID.ToString(), 
                "SERVER=(local);DATABASE=northwind;Trusted_Connection=Yes");
            _data = new DataTable();

            adapter.Fill(_data);
        }

        // ************************************************************************
        // Build the Web part's UI
        private void BuildUI(HtmlTextWriter writer)
        {
            RenderQueryStructure (writer);
            RenderEmployeeInfo (writer);
        }

        // ************************************************************************
        // Render the query structure 
        private void RenderQueryStructure(HtmlTextWriter writer)
        {
            writer.Write("Angestellten-ID eingeben: ");
            txt.RenderControl(writer);   
            btn.RenderControl(writer);   
            writer.Write ("<hr size=1>");
        }

        // ************************************************************************
        // Event handler for the button's Click event
        private void OnFindEmployee(Object sender, EventArgs e)
        {
            _empID = Convert.ToInt32(txt.Text);
        }

        // ************************************************************************
        // Render the employee information
        private void RenderEmployeeInfo(HtmlTextWriter writer)
        {
            Table _table = new Table ();

            // ID field
            TableRow id = new TableRow ();
            TableCell id_cell1 = new TableCell ();

            id_cell1.VerticalAlign = VerticalAlign.Top;
            id_cell1.Text = "<b>ID<b>";
            id.Cells.Add (id_cell1);

            TableCell id_cell2 = new TableCell ();

            id_cell2.Text = _data.Rows[0]["employeeid"].ToString ();
            id.Cells.Add (id_cell2);
            _table.Rows.Add (id);

            // Name field
            TableRow name = new TableRow ();
            TableCell name_cell1 = new TableCell ();

            name_cell1.VerticalAlign = VerticalAlign.Top;
            name_cell1.Text = "<b>Name<b>";
            name.Cells.Add (name_cell1);

            TableCell name_cell2 = new TableCell ();

            name_cell2.Text = String.Format ("{0}, {1}", _data.Rows[0]["lastname"].ToString (), _data.Rows[0]["firstname"].ToString ());
            name.Cells.Add (name_cell2);
            _table.Rows.Add (name);

            // Title field
            TableRow title = new TableRow ();
            TableCell title_cell1 = new TableCell ();

            title_cell1.VerticalAlign = VerticalAlign.Top;
            title_cell1.Text = "<b>Position<b>";
            title.Cells.Add (title_cell1);

            TableCell title_cell2 = new TableCell ();

            title_cell2.Text = _data.Rows[0]["title"].ToString ();
            title.Cells.Add (title_cell2);
            _table.Rows.Add (title);

            // Notes field
            TableRow notes = new TableRow ();
            TableCell notes_cell1 = new TableCell ();

            notes_cell1.VerticalAlign = VerticalAlign.Top;
            notes_cell1.Text = "<b>Hinweise<b>";
            notes.Cells.Add (notes_cell1);

            TableCell notes_cell2 = new TableCell ();

            notes_cell2.Text = _data.Rows[0]["notes"].ToString ();
            notes.Cells.Add (notes_cell2);
            _table.Rows.Add (notes);
            _table.RenderControl (writer);

        }
    }
}

