using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Drawing;
using System.Collections;
using System.Drawing.Design;
using ProAspNet20.Advanced.CS.Components.Design;




namespace ProAspNet20.Advanced.CS.Components
{
    [DefaultProperty("Value")]
    [ToolboxData("<{0}:GaugeBar runat=\"server\" width=\"100px\" height=\"20px\" />")]
    [Designer(typeof(GaugeBarDesigner))]
    public class GaugeBar : DataBoundControl
    {
        private TextItemStyle _textStyle;
        private int _dividerCell;
        private GaugeBarDataItem _dataItem;
  

        // *******************************************************
        // Ctor
        public GaugeBar()
        {
            BackColor = Color.White;
            ForeColor = Color.LawnGreen;
            BorderStyle = BorderStyle.Solid;
            BorderWidth = Unit.Pixel(1);
            BorderColor = Color.Black;
            Width = Unit.Pixel(100);
            Height = Unit.Pixel(20);
        }

        #region IE-only Filter Properties
        private FilterSettings _gaugeFilters = null;
        [Category("Appearance")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [Description("Definiert die Filtereinstellungen fr die Statusanzeige")]
        [Editor(typeof(GaugeFilterSettingsEditor), typeof(UITypeEditor))]
        public FilterSettings Filter
        {
            get
            {
                if (_gaugeFilters == null)
                    _gaugeFilters = new FilterSettings();
                return _gaugeFilters;
            }
        }
        #endregion


        #region Properties
        // *******************************************************
        // Ruft den in der Statusanzeige darzustellenden Wert ab oder legt diesen fest
        [Category("Behavior")]
        [Description("Ruft den in der Statusanzeige darzustellenden Wert ab oder legt diesen fest")]
        [Themeable(false)]
        public float Value
        {
            get
            {
                object o = ViewState["Value"];
                if (o == null)
                    return 0;
                return (float) o;
            }
            set 
            {
                ViewState["Value"] = value;
                if (value > Maximum)
                    ViewState["Value"] = Maximum;
            }
        }

        // *******************************************************
        // Ruft den in der Statusanzeige darzustellenden Maximalwert ab oder legt diesen fest
        [Category("Behavior")]
        [Themeable(false)]
        [Description("Ruft den in der Statusanzeige darzustellenden Maximalwert ab oder legt diesen fest")]
        public float Maximum
        {
            get
            {
                object o = ViewState["Maximum"];
                if (o == null)
                    return 100;
                return (float) o;
            }
            set { ViewState["Maximum"] = value; }
        }

        // *******************************************************
        // Anzahl der Segmente, in die die Statusanzeige unterteilt werden soll
        [Category("Layout")]
        [Themeable(false)]
        [Description("Ruft die Anzahl der Markierungen (Einkerbungen) ab, die entlang der Statusanzeige angezeigt werden sollen, oder legt diese fest")]
        public int Segments
        {
            get
            {
                object o = ViewState["Segments"];
                if (o == null)
                    return 4;
                return (int) o;
            }
            set 
            { 
                ViewState["Segments"] = value;
                if( value < 1)
                    ViewState["Segments"] = 1; 
            }
        }

        // *******************************************************
        // Ruft das Muster zur Formatierung des in der Statusanzeige dargestellten Wertes ab, oder legt dieses fest
        [Bindable(false)]
        [Category("Appearance")]
        [Description("Ruft das Muster zur Formatierung des in der Statusanzeige dargestellten Wertes ab, oder legt dieses fest")]
        [Themeable(true)]
        [Localizable(true)]
        public string FormatString
        {
            get
            {
                object o = ViewState["FormatString"];
                if (o == null)
                    return "<b>{0}</b> / <b>{1}</b>";
                return (string) o;
            }
            set { ViewState["FormatString"] = value; }
        }

        // *******************************************************
        // Gibt an, ob Markierungen (Einkerbungen) angezeigt werden sollen
        [Category("Appearance")]
        [Themeable(true)]
        [Description("Gibt an, ob Markierungen (Einkerbungen) angezeigt werden sollen")]
        public bool GridLines
        {
            get
            {
                object o = ViewState["GridLines"];
                if (o == null)
                    return true;
                return (bool) o;
            }
            set { ViewState["GridLines"] = value; }
        }


        // ***************************************************************
        // Ruft den Stil der Statusanzeigenbeschriftung ab oder legt diesen fest
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Appearance")]
        [Description("Ruft den Stil der Statusanzeigenbeschriftung ab oder legt diesen fest")]
        [Themeable(true)]
        public TextItemStyle TextStyle
        {
            get
            {
                if (_textStyle == null)
                    _textStyle = new TextItemStyle();

                if (IsTrackingViewState)
                    ((IStateManager)_textStyle).TrackViewState();

                return _textStyle;
            }
        }
        #endregion


        #region Data-bound properties
        // ***************************************************************
        // Gets and sets the name of the field bound to the gauge value
        [Category("Data")]
        [Themeable(false)]
        [Description("Gets and sets the name of the field bound to the gauge value")]
        public virtual string DataValueField
        {
            get
            {
                object o = ViewState["DataValueField"];
                if (o == null)
                    return String.Empty;
                return (string) o;
            }
            set { ViewState["DataValueField"] = value; }
        }


        // ***************************************************************
        // Gets and sets the name of the field bound to the gauge text
        [Category("Data")]
        [Description("Gets and sets the name of the field bound to the gauge text")]
        [Themeable(false)]
        public virtual string DataTextField
        {
            get
            {
                object o = ViewState["DataTextField"];
                if (o == null)
                    return String.Empty;
                return (string) o;
            }
            set { ViewState["DataTextField"] = value; }
        }


        // ***************************************************************
        // Represent the internally bound data item object
        [Browsable(false)]
        [Themeable(false)]
        public GaugeBarDataItem DataItem
        {
            get
            {
                if (_dataItem == null)
                {
                    _dataItem = new GaugeBarDataItem();
                    if (base.IsTrackingViewState)
                        _dataItem.TrackViewState();
                }
                return _dataItem;
            }
        }

        #endregion


        #region ViewState management
        protected override void LoadViewState(object savedState)
        {
            if (savedState != null)
            {
                Triplet t = (Triplet) savedState;
                base.LoadViewState(t.First);
                DataItem.LoadViewState(t.Second);
                TextStyle.LoadViewState(t.Third);
            }
            else
            {
                base.LoadViewState(null);
            }
        }

        protected override object SaveViewState()
        {
            // Get the standard state objectViewState container
            object baseState = base.SaveViewState();

            // Get the state object for the DataItem property
            object itemState = DataItem.SaveViewState();

            // Get the state object for the TextStyle object
            object styleState = TextStyle.SaveViewState();

            // Pack everything into a unique object
            return new Triplet(baseState, itemState, styleState);
        }
        #endregion


        #region Binding
        protected override void PerformDataBinding(System.Collections.IEnumerable data)
        {
            if (data == null)
                return;

            IEnumerator e = data.GetEnumerator();
            e.MoveNext();

            float displayValue = -1;  
            string displayText = String.Empty; 
            if (!String.IsNullOrEmpty(DataValueField))
                displayValue = (float) DataBinder.GetPropertyValue(e.Current, DataValueField);

            if (!String.IsNullOrEmpty(DataTextField))
                displayText = (string) DataBinder.GetPropertyValue(e.Current, DataTextField);

            DataItem.Value = displayValue;
            DataItem.Text = displayText;
        }
        #endregion


        #region Rendering
        // *******************************************************
        // Renders the output of the control
        protected override void Render(HtmlTextWriter output)
        {   
            // Ensure it renders at design-time without custom designers
            if (DesignMode)
            {
                RecreateChildControls();
            }

            PrepareControlForRendering();
            base.RenderContents(output);
        }
        
        // *******************************************************
        // Recreate child controls (Design-time relevance)
        protected virtual void RecreateChildControls()
        {
            base.ChildControlsCreated = false;
            this.EnsureChildControls();
        }
 
        // *******************************************************
        // Style the control 
        protected virtual void PrepareControlForRendering()
        {
            // Make sure there are controls to work with
            if (Controls.Count < 1)
                return;

            // Get the table
            Table outer = (Table)Controls[0];
            Table t = (Table) outer.Rows[0].Cells[0].Controls[0];

            // Apply the table style
            t.CopyBaseAttributes(this);
            if (ControlStyleCreated)
                t.MergeStyle(ControlStyle);

            // Configure additional property on the overall table
            t.CellPadding = 0;
            t.CellSpacing = 0;


            // Set colors on the cells
            for (int i = 0; i < Segments; i++)
            {
                TableCell cell = t.Rows[0].Cells[i];
                if (GridLines)
                {
                    cell.BorderColor = BorderColor;
                    cell.BorderStyle = BorderStyle;
                    cell.BorderWidth = BorderWidth;
                }

                if (i < _dividerCell)
                    cell.BackColor = ForeColor;
                if (i >= _dividerCell)
                    cell.BackColor = BackColor;
                if (i == _dividerCell)
                {
                    Table inner = (Table) cell.Controls[0];
                    if (inner.Rows[0].Cells.Count > 1)
                    {
                        inner.Rows[0].Cells[0].BackColor = ForeColor;
                        inner.Rows[0].Cells[1].BackColor = BackColor;
                    }
                    else
                    {
                        // ratio in the divider cell is 0-100 
                        // (i.e., a value of 25 on 100 with 4 segments)
                        inner.Rows[0].Cells[0].BackColor = BackColor;
                    }
                }
            }

            // Style the gauge bar text style
            if (!_textStyle.DisplayTextAtBottom)
            {
                // extra cell
                outer.Rows[0].Cells[1].MergeStyle(TextStyle);
            }
            else if (_textStyle.RenderInsideTable && _textStyle.DisplayTextAtBottom)
            {
                // bottom row
                TableRow row = t.Rows[1];
                row.MergeStyle(TextStyle);
            }
            else
            {
                // text below
                Label lbl = (Label)Controls[1];
                lbl.MergeStyle(TextStyle);
            }

            // Apply filters
            if (Filter.FilterType == GaugeFilterType.Glow)
                ApplyFilter(outer);
            else
                ApplyFilter(t);

        }

        // *******************************************************
        // Control the creation of child controls
        protected override void CreateChildControls()
        {
 	        base.CreateChildControls();
            Controls.Clear();
            CreateControlHierarchy();
            ClearChildViewState();
        }

        // *******************************************************
        // Control the creation of child controls
        protected virtual void CreateControlHierarchy()
        {
            // Build the outermost container table
            Table outer = new Table();
            TableRow outerRow = new TableRow();
            outer.Rows.Add(outerRow);

            // Ruler cell
            TableCell rulerCell = new TableCell();
            outerRow.Cells.Add(rulerCell);
            BuildGaugeBar(rulerCell);

            // Text cell
            TableCell textCell = new TableCell();
            if (!_textStyle.DisplayTextAtBottom)
            {
                outerRow.Cells.Add(textCell);
                BuildLabel(textCell);
            }

            // Save the control tree
            Controls.Add(outer);

            // Build the label
            if (!_textStyle.RenderInsideTable && _textStyle.DisplayTextAtBottom)
                BuildLabel(null);
        }

        // *******************************************************
        // Build the control tree for the whole gauge bar 
        void BuildGaugeBar(TableCell container)
        {
            // Create the table with one or two rows: ruler (and label)
            Table t = new Table();
            TableRow ruler = new TableRow();
            t.Rows.Add(ruler);

            // Build the ruler row
            BuildRuler(ruler);

            // Build the label
            if (_textStyle.RenderInsideTable)
                BuildLabelIntoTable(t);

            // Save the control tree
            container.Controls.Add(t);
        }

        // *******************************************************
        // Build the control tree for the ruler 
        void BuildRuler(TableRow ruler)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();
            float valueToRepresent = 100f * buf / Maximum;
            int numOfSegments = GetNumOfSegments();

            int segmentWidth = 100/numOfSegments;
			bool finished = false;

            for (int i = 1; i <= numOfSegments; i++)
            {
                if (valueToRepresent < i * segmentWidth)
                {
                    if (finished)
                    {
                        // Still-To-Do
                        TableCell stillToDo = new TableCell();
                        ruler.Cells.Add(stillToDo);
                        stillToDo.Width = Unit.Percentage(segmentWidth);
                    }
                    else
                    {
                        // Cell to divide
                        _dividerCell = i-1;  // need a 0-based index
                        TableCell cell = new TableCell();
                        ruler.Cells.Add(cell);
                        cell.Width = Unit.Percentage(segmentWidth);
                        cell.Height = Unit.Percentage(100);

                        // Add a child table to the cell
                        Table child = new Table();
                        child.Width = Unit.Percentage(100);
                        child.Height = Unit.Percentage(100);
                        cell.Controls.Add(child);
                        child.CellPadding = 0;
                        child.CellSpacing = 0;
                        TableRow childRow = new TableRow();
                        child.Rows.Add(childRow);
                        
						float fx = (100*(valueToRepresent-segmentWidth*(i-1))/segmentWidth);
                        if (valueToRepresent > (i - 1) * segmentWidth)
                        {
                            TableCell left = new TableCell();
                            childRow.Cells.Add(left);
                            left.Width = Unit.Percentage(fx);
                        }
                        TableCell right = new TableCell();
                        childRow.Cells.Add(right);
                        right.Width = Unit.Percentage(100 - fx);
                        finished = true;
                    }
                }
                else
                {
                    // Done
                    TableCell done = new TableCell();
                    ruler.Cells.Add(done);
                    done.Width = Unit.Percentage(segmentWidth);
                }
            }
        }

        // *******************************************************
        // Build the control tree for the label 
        void BuildLabel(TableCell container)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();

            // Get the string to display on the label 
            string msg = GetTextToRepresent();

            Label lbl = new Label();
            if (container is TableCell)
                container.Controls.Add(lbl);
            else
                Controls.Add(lbl);
            lbl.Text = String.Format(msg, buf, Maximum);
        }

        // *******************************************************
        // Build the control tree for the label 
        void BuildLabelIntoTable(Table t)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();
            int numOfSegments = GetNumOfSegments();

            // Get the string to display on the label 
            string msg = GetTextToRepresent();

            if (_textStyle.DisplayTextAtBottom)
            {
                // Add a bottom row
                TableRow label = new TableRow();
                t.Rows.Add(label);

                TableCell lblCell = new TableCell();
                label.Cells.Add(lblCell);

                lblCell.ColumnSpan = numOfSegments;
                lblCell.Text = String.Format(msg, buf, Maximum);
            }
        }

        #region Helpers
        // *******************************************************
        // Helper function
        float GetValueToRepresent()
        {
            float f = 0;
            if (DataItem.Value >= 0)
                f = DataItem.Value;
            else
                f = Value;

            return f;
        }

        // *******************************************************
        // Helper function
        string GetTextToRepresent()
        {
            string msg = "";
            if (!String.IsNullOrEmpty(DataItem.Text))
                msg = DataItem.Text;
            else
                msg = FormatString;
            return msg;
        }

        // *******************************************************
        // Helper function
        int GetNumOfSegments()
        {
            return Segments;
        }
        #endregion

        #endregion


        #region Filter rendering
        // *******************************************************
        // Apply IE DHTML filters to the row 
        private void ApplyFilter(Table t)
        {
            // No filter requested
            GaugeFilterType filter = Filter.FilterType;
            if (filter == GaugeFilterType.None)
                return;

            string style = "";
            if (filter == GaugeFilterType.Alpha)
            {
                string alpha = "progid:DXImageTransform.Microsoft.Alpha(style=2,opacity={0},finishOpacity=100,startX=0,finishX=100,startY=100,finishY=0)";
                style = String.Format(alpha, Filter.AlphaOpacity);
            }
            else if (filter == GaugeFilterType.Glow)
            {
                string glow = "progid:DXImageTransform.Microsoft.Glow(color={0},strength={1})";
                style = String.Format(glow, ColorTranslator.ToHtml(Filter.GlowColor),
                    Filter.GlowStrength);
            }

            t.Style["Filter"] = style;
            return;
        }
        #endregion

    }



    #region CLASS::GaugeBarDataItem
    public class GaugeBarDataItem : IStateManager
    {
        private string _text = "";
        private float _value = -1;
        private bool _marked;

        #region IStateManager
        public bool IsTrackingViewState
        {
            get { return _marked; }
        }

        public void LoadViewState(object state)
        {
            if (state != null)
            {
                Pair p = (Pair)state;
                _value = (float)p.First;
                _text = (string)p.Second;
            }
        }

        public object SaveViewState()
        {
            return new Pair(_value, _text);
        }

        public void TrackViewState()
        {
            _marked = true;
        }
        #endregion

        public GaugeBarDataItem()
        {
        }
        public GaugeBarDataItem(float value, string text)
        {
            _text = text;
            _value = value;
        }
        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }
        public float Value
        {
            get { return _value; }
            set { _value = value; }
        }
    }
    #endregion
}
