using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;
using System.ComponentModel;

namespace ProAspNet20.Advanced.CS.Components
{
    public class TextItemStyle : TableItemStyle, IStateManager
    {
        #region Private Members
        private bool _renderInsideTable;
        private bool _displayTextAtBottom;
        #endregion

        public TextItemStyle()
        {
            _displayTextAtBottom = true;
            _renderInsideTable = false;
        }

        // Public Members
        [NotifyParentProperty(true)]
        public bool RenderInsideTable
        {
            get { return _renderInsideTable; }
            set { _renderInsideTable = value; }
        }

        [NotifyParentProperty(true)]
        public bool DisplayTextAtBottom
        {
            get { return _displayTextAtBottom; }
            set { _displayTextAtBottom = value; }
        }

        #region IStateManager
        bool IStateManager.IsTrackingViewState
        {
            get
            {
                return base.IsTrackingViewState;
            }
        }
        void IStateManager.TrackViewState()
        {
            base.TrackViewState();
        }
        object IStateManager.SaveViewState()
        {
            return SaveViewState();
        }
        void IStateManager.LoadViewState(object state)
        {
            LoadViewState(state);
        }
        #endregion

        #region Other members
        public new object SaveViewState()
        {
            object[] state = new object[2];
            state[0] = base.SaveViewState();

            object[] extraData = new object[2];
            extraData[0] = _renderInsideTable;
            extraData[1] = _displayTextAtBottom;
            state[1] = (object)extraData;
            
            return state;
        }

        public new void LoadViewState(object state)
        {
            if (state == null)
                return;
            object[] myState = (object[])state;
            base.LoadViewState(myState[0]);

            object[] extraData = (object[])myState[1];
            _renderInsideTable = (bool)extraData[0];
            _displayTextAtBottom = (bool)extraData[1];
        }
        #endregion
    }
}
