using System;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.ComponentModel;



namespace ProAspNet20.Advanced.CS.Components
{
    public class Panel : System.Web.UI.WebControls.Panel, INamingContainer
    {
        #region New Properties
        // Property: EnableDropDown
        // Gets or sets whether the panel can be collapsed or expanded
        [Themeable(false)]
        public bool EnableDropDown 
        {
            get
            {
                object o = ViewState["EnableDropDown"];
                if (o == null)
                    return false;
                return (bool) o;
            }
            set { ViewState["EnableDropDown"] = value; }
        }

        // Property: EnableClientScript
        // Gets or sets whether the panel should use client script
        [Themeable(false)]
        public bool EnableClientScript
        {
            get
            {
                object o = ViewState["EnableClientScript"];
                if (o == null)
                    return false;
                return (bool)o;
            }
            set { ViewState["EnableClientScript"] = value; }
        }

        // Property: ShowExpanded
        // Gets or sets whether the panel should be expanded
        [Themeable(false)]
        public bool ShowExpanded
        {
            get
            {
                object o = ViewState["ShowExpanded"];
                if (o == null)
                    return true;
                return (bool) o;
            }
            set 
            { 
                ViewState["ShowExpanded"] = value;
            }
        }

        // Property: Caption
        // Gets or sets the text of the panel
        [Themeable(false)]
        public string Caption
        {
            get
            {
                object o = ViewState["Caption"];
                if (o == null)
                    return "Panel";
                return (string)o;
            }
            set { ViewState["Caption"] = value; }
        }

        // Property: CaptionBackColor
        // Gets or sets the background color of the caption
        public Color CaptionBackColor
        {
            get
            {
                object o = ViewState["CaptionBackColor"];
                if (o == null)
                    return Color.SkyBlue;
                return (Color)o;
            }
            set { ViewState["CaptionBackColor"] = value; }
        }

        // Property: CaptionForeColor
        // Gets or sets the foreground color of the caption
        public Color CaptionForeColor
        {
            get
            {
                object o = ViewState["CaptionForeColor"];
                if (o == null)
                    return Color.White;
                return (Color)o;
            }
            set { ViewState["CaptionForeColor"] = value; }
        }
        #endregion


        #region New Events
        public event EventHandler<PanelClickEventArgs> PanelClick;
        #endregion


        #region Control State
        private bool _panelDisplayed;
        protected override object SaveControlState()
        {
            Pair p = new Pair();
            p.First = base.SaveControlState();
            p.Second = _panelDisplayed;
            return p;
        }

        protected override void LoadControlState(object savedState)
        {
            if (savedState == null)
                return;

            Pair p = (Pair) savedState;
            base.LoadControlState(p.First);
            _panelDisplayed = (bool)p.Second;
        }
        #endregion

        
        #region Initialization
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            Page.RegisterRequiresControlState(this);
        }


        protected override void OnLoad(EventArgs e)
        {
            if (!Page.IsPostBack)
                _panelDisplayed = ShowExpanded;
        }
        #endregion


        #region Style

        // *******************************************************************************************
		// PrepareControlForRendering 
        protected virtual void PrepareControlForRendering()
        {
            // Make sure there are controls to work with
            if (Controls.Count != 1)
                return;

            // Apply the table style
            Table t = (Table) Controls[0];
            t.CopyBaseAttributes(this);
            if (ControlStyleCreated)
                t.ApplyStyle(ControlStyle);

            // cell spacing/padding set on title work for the whole table
            t.CellPadding = 1;
            t.CellSpacing = 0;

            // Style the title row
            TableRow row1 = t.Rows[0];
            row1.BackColor = CaptionBackColor;
            row1.ForeColor = CaptionForeColor;
        }
        #endregion


        #region Rendering
        protected override void CreateChildControls()
        {
            if (EnableDropDown)
            {
                base.CreateChildControls();
                CreateControlHierarchy();
            }
            else
                base.CreateChildControls();
        }

        protected override void Render(HtmlTextWriter writer)
        {
            PrepareControlForRendering();
            base.Render(writer);
        }

        protected virtual void CreateControlHierarchy()
        {
            Table t = new Table();

            TableRow row1 = new TableRow();
            t.Rows.Add(row1);

            TableCell cell1 = new TableCell();
            row1.Cells.Add(cell1);
            cell1.Text = "&nbsp;" + Caption;
            
            TableCell cell2 = new TableCell();
            row1.Cells.Add(cell2);
            cell2.HorizontalAlign = HorizontalAlign.Right;

            TableRow row2 = new TableRow();
            t.Rows.Add(row2);
            TableCell body = new TableCell();
            body.ID = "Body";
            row2.Cells.Add(body);
            body.ColumnSpan = 2;

            Control[] rg = new Control[Controls.Count];
            Controls.CopyTo(rg, 0);

            foreach (Control ctl in rg)
                body.Controls.Add(ctl);

            Controls.Clear();
            Controls.Add(t);


            WebControl img;
            if (!EnableClientScript)
            {
                img = new ImageButton();
                ((ImageButton)img).Click += new ImageClickEventHandler(OnClick);
                cell2.Controls.Add(img);
            }
            else
            {
                img = new System.Web.UI.WebControls.Image();
                img.ID = "Icon";
                cell2.Controls.Add(img);

                // Add script code here
                img.Attributes["onmouseover"] = "this.style.cursor = \"hand\";";
                img.Attributes["onmouseout"] = "this.style.cursor = \"\";";
                img.Attributes["onclick"] = "__toggle()";
                if (!Page.ClientScript.IsClientScriptBlockRegistered("__toggle"))
                {
                    string js = BuildScript(body);
                    Page.ClientScript.RegisterClientScriptBlock(this.GetType(), "__toggle", js, true);
                }
            }

            
            ShowChildControls(_panelDisplayed);
            ShowImageButton(_panelDisplayed);

            return;
        }

        private void ShowChildControls(bool display)
        {
            // Make sure there are controls to work with
            if (Controls.Count != 1)
                return;

            // Apply the table style
            Table t = (Table)Controls[0];
            TableRow r = t.Rows[1];
            TableCell body = r.Cells[0];
            body.Style["display"] = (display ? "" : "none");
        }

        private void ShowImageButton(bool display)
        {
            // Make sure there are controls to work with
            if (Controls.Count != 1)
                return;

            // Apply the table style
            Table t = (Table)Controls[0];
            TableRow r = t.Rows[0];
            TableCell icon = r.Cells[1];

            // Can be Image or ImageButton depending on EnableClientScript. ImageButton derives from Image
            System.Web.UI.WebControls.Image img = (System.Web.UI.WebControls.Image)icon.Controls[0];

            string imageName = "ProAspNet20.Advanced.CS.Components.Expand.bmp";
            if (display && !EnableClientScript)
                imageName = "ProAspNet20.Advanced.CS.Components.Collapse.bmp";
            img.ImageUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), imageName);
        }

        private string BuildScript(TableCell body)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("function __toggle() {");
            sb.AppendFormat("   var body = document.getElementById(\"{0}\");\r\n", body.ClientID);
            sb.AppendLine("   var display = body.style.display;");
            sb.AppendLine("   if (display == \"\") {");
            sb.AppendLine("      body.style.display = \"none\";");
            sb.AppendLine("   } else {");
            sb.AppendLine("      body.style.display = \"\";");
            sb.AppendLine("   }");
            sb.AppendLine("}");

            return sb.ToString();
        }

        #endregion


        #region Clicking

        private void OnClick(object sender, ImageClickEventArgs e)
        {
            PanelClickEventArgs args = new PanelClickEventArgs();
            args.BeingClosed = _panelDisplayed;
            OnPanelClick(args);

            // Must update _panelDisplayed to reflect the click
            _panelDisplayed = !_panelDisplayed;

            ShowChildControls(_panelDisplayed);
            ShowImageButton(_panelDisplayed);
        }


        protected virtual void OnPanelClick(PanelClickEventArgs args)
        {
            if (PanelClick != null)
                PanelClick(this, args);
        }
        #endregion
    }


    public class PanelClickEventArgs : EventArgs
    {
        public bool BeingClosed;
    }

}
