#region Using directives
using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Data;
using System.Collections.Specialized;
using System.IO;
using System.Collections;
using System.Xml;
using System.Text;
using System.Web.UI.HtmlControls;
#endregion


namespace ProAspNet20.Advanced.CS.Components
{
    #region Class PictureViewerInfo
    class PictureViewerInfo
    {
        public PictureViewerInfo()
        {
            DisplayWidth = 200;
            ColumnCount = 3;
        }
        public int DisplayWidth;
        public int ColumnCount;
        public string FolderName;
    }
    #endregion


    public class PictureViewerHandler : IHttpHandler
    {
        #region IHttpHandler 
        // Override the ProcessRequest method
		public void ProcessRequest(HttpContext context)
		{
            PictureViewerInfo info = GetFolderInfo(context);
            string html = CreateOutput(info);

			// Output the data
            // <meta http-equiv="Page-Exit" content="progid:DXImageTransform.Microsoft.Fade(duration=.5)" /> 

			context.Response.Write("<html><head>");
			context.Response.Write("<title>Picture Web Viewer</title>");
			context.Response.Write("</head><body>");

            context.Response.Write(html);

            context.Response.Write("</body></html>");
		}

		// Override the IsReusable property
		public bool IsReusable
		{
			get { return false; }
        }
        #endregion


        #region Helper methods
        private void ErrorMessage(HttpContext context, string msg)
        {
            context.Response.Write(msg);
            context.Response.End();
        }

        private PictureViewerInfo GetFolderInfo(HttpContext context)
        {
            // Get optional parameters: Width and Cols
            PictureViewerInfo info = new PictureViewerInfo();
            object p1 = context.Request.Params["Width"];
            object p2 = context.Request.Params["Cols"];
            if (p1 != null)
                Int32.TryParse((string)p1, out info.DisplayWidth);
            if (p2 != null)
                Int32.TryParse((string)p2, out info.ColumnCount);

            // Gets the segment that precedes "view.axd"
            string fileName = "";

            fileName = context.Request.Path.ToString();
            fileName = fileName.Replace("view.axd", "");
            if (fileName.EndsWith("/") || fileName.EndsWith(@"\"))
            {
                fileName = fileName.Substring(0, fileName.Length - 1);
                fileName = fileName.TrimEnd();
            }
            try
            {
                fileName = context.Server.MapPath(fileName);
            }
            catch (Exception e)
            {
                ErrorMessage(context, e.Message);
            }

            info.FolderName = fileName;
            return info;
        }

        private string CreateOutput(PictureViewerInfo info)
        {
            string pathName = info.FolderName;
            DirectoryInfo di = new DirectoryInfo(pathName);
            if (di.Exists)
                return CreateOutputForFolder(info, di);
            else
                return CreateOutputForSingleImage(pathName);
        }

        private string CreateOutputForFolder(PictureViewerInfo info, DirectoryInfo di) 
        {
            ArrayList images = GetAllImages(di);

            Table t = new Table();

            int index = 0;
            bool moreImages = true;
            while (moreImages)
            {
                TableRow row = new TableRow();
                t.Rows.Add(row);

                for (int i = 0; i < info.ColumnCount; i++)
                {
                    TableCell cell = new TableCell();
                    row.Cells.Add(cell);

                    System.Web.UI.WebControls.Image img = new System.Web.UI.WebControls.Image();
                    FileInfo fi = (FileInfo)images[index];
                    img.ImageUrl = fi.Name;
                    img.Width = Unit.Pixel(info.DisplayWidth);

                    HtmlAnchor a = new HtmlAnchor();
                    a.HRef = fi.Name;
                    a.Controls.Add(img); 


                    cell.Controls.Add(a);

                    index++;
                    moreImages = (index < images.Count);
                    if (!moreImages)
                        break;
                }
            }

            string html = Utils.RenderControlAsString(t);
            return html; 
        }

        private ArrayList GetAllImages(DirectoryInfo di)
        {
            string[] fileTypes = { "*.bmp", "*.gif", "*.jpg", "*.png" };
            ArrayList images = new ArrayList();
            foreach (string ext in fileTypes)
            {
                FileInfo[] files = di.GetFiles(ext);
                if (files.Length > 0)
                {
                    images.AddRange(files);
                }
            }
            return images;
        }

        private string CreateOutputForSingleImage(string path)
		{
            System.Web.UI.WebControls.Image img = new System.Web.UI.WebControls.Image();
            img.ImageUrl = path;
            string html = Utils.RenderControlAsString(img);
            return html;
        }
        #endregion
    }
}
