#region Using directives
using System;
using System.Drawing;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Data;
using System.Collections.Specialized;
using System.IO;
using System.Data.SqlClient;
using System.Xml;
using System.Web.Hosting;
#endregion


namespace ProAspNet20.Advanced.CS.Components
{
    #region Class SqlxData
    class SqlxData 
	{
		public string ConnectionString;
		public string QueryText;
    }
    #endregion


    public class QueryHandler : IHttpHandler
    {
        #region IHttpHandler 
        // Override the ProcessRequest method
		public void ProcessRequest(HttpContext context)
		{
			// Parses the SQLX file 
			SqlxData[] data = ParseFile(context);

			// Create the output as HTML
			StringCollection htmlColl = CreateOutput(data);

			// Output the data
			context.Response.Write("<html><head><title>");
			context.Response.Write("QueryHandler Output");
			context.Response.Write("</title></head><body>");
            foreach (string html in htmlColl)
            {
                context.Response.Write(html);
                context.Response.Write("<hr />");
            }
			context.Response.Write("</body></html>");
		}

		// Override the IsReusable property
		public bool IsReusable
		{
			get { return false; }
        }
        #endregion


        #region Helper methods
        private SqlxData[] ParseFile(HttpContext context)
        {
            XmlDocument doc = new XmlDocument();
            using (Stream file = VirtualPathProvider.OpenFile(context.Request.Path))
            {
                doc.Load(file);
            }

            // Visit the <mapping> nodes
            XmlNodeList mappings = doc.SelectNodes("queries/query");
            SqlxData[] descriptors = new SqlxData[mappings.Count];
            for (int i=0; i < descriptors.Length; i++)
            {
                XmlNode mapping = mappings[i];

                SqlxData query = new SqlxData();
                descriptors[i] = query;

                try
                {
                    query.ConnectionString = mapping.Attributes["connString"].Value;
                    query.QueryText = mapping.InnerText;
                }
                catch
                {
                    context.Response.Write("Error parsing the input file.");
                    context.Response.End();
                }
            }

            return descriptors;
        }

		private StringCollection CreateOutput(SqlxData[] descriptors)
		{
            StringCollection coll = new StringCollection();

            foreach (SqlxData data in descriptors)
            {
                // Run the query
                DataTable dt = new DataTable();
                SqlDataAdapter adapter = new SqlDataAdapter(data.QueryText, data.ConnectionString);
                adapter.Fill(dt);

                // Prepare the grid
                DataGrid grid = new DataGrid();
                grid.CellPadding = 2;
                grid.Font.Name = "verdana";
                grid.Font.Size = FontUnit.Point(8);
                grid.HeaderStyle.BackColor = Color.SkyBlue;
                grid.HeaderStyle.Font.Size = FontUnit.Larger;
                grid.HeaderStyle.Font.Bold = true;
                grid.DataSource = dt;
                grid.DataBind();

                // Get the HTML
                string html = Utils.RenderControlAsString(grid);
                coll.Add(html); 
            }

			return coll;
        }
        #endregion
    }
}
