using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;


namespace ProAspNet20.Advanced.CS.Components 
{
    public class SafeListPageFilter : PageParserFilter  
    {
        private List<Type> _safeControlList;
        protected virtual List<Type> SafeControlList
        {
            get
            {
                if (_safeControlList == null)
                {
                    _safeControlList = new List<Type>();
                    LoadSafeControlList();
                }
                return _safeControlList;
            }
        }

        public override bool AllowBaseType(Type baseType)
        {
            // Check the base page type.
            // Note that base.AllowBaseType(baseType) returns FALSE
            return true;  
        }

        public override bool AllowControl(Type controlType, ControlBuilder builder)
        {
            //return SafeControlList.Contains(controlType);
            return true;
        }        
        
        private void LoadSafeControlList()
        {
            SafeControlList.Add(typeof(System.Web.UI.Page));
            SafeControlList.Add(typeof(System.Web.UI.HtmlControls.HtmlForm));
            SafeControlList.Add(typeof(System.Web.UI.HtmlControls.HtmlHead));
            SafeControlList.Add(typeof(System.Web.UI.WebControls.TextBox));
            SafeControlList.Add(typeof(System.Web.UI.WebControls.Button));
        }


        #region Properties
        public override bool AllowCode
        {
            get { return true; }
        }

        public override int NumberOfControlsAllowed
        {
            get { return 20; }
        }

        public override int NumberOfDirectDependenciesAllowed
        {
            get { return 10; }
        }

        public override int TotalNumberOfDependenciesAllowed
        {
            get { return 100; }
        }
        #endregion

        #region Virtual References
        public override bool AllowVirtualReference(string refVirtualPath, VirtualReferenceType refType)
        {
            // add the code to check for a user control so that a user control cannot be used
            // in the page	
            if (refType == VirtualReferenceType.UserControl)
                return false;
            else
	            return true;
        }
        #endregion

        #region ParseComplete
        public override void ParseComplete(ControlBuilder rootBuilder)
        {
            if (rootBuilder is FileLevelPageControlBuilder) 
                rootBuilder.AppendLiteralString("<b>written by Dino Esposito</b>");
        }
        #endregion
    }
}
