using System;
using System.ComponentModel;
using System.Collections;
using System.Drawing;
using System.Web.UI;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;


namespace ProAspNet20.Advanced.CS.Components
{
    public sealed class StockViewerWebPart : WebPart, ICallbackEventHandler
    {
        #region Private Members
		private string _stocks;
		private int _refreshRate;
		private Random _rnd;
		private WebPartVerbCollection _verbs;
        #endregion


        #region ICallbackEventHandler
        private string _results;
        void ICallbackEventHandler.RaiseCallbackEvent(string argument)
        {
            string buf = "," + _stocks;
			string[] list = buf.Split(new char[] { ',' });
			list[0] = DateTime.Now.ToString("hh:mm:ss");
            for (int i=1; i<list.Length; i++)
            {
                list[i] = GetCurrentValue(list[i]);
            }
            _results = String.Join(",", list);
            return;
        }
        string ICallbackEventHandler.GetCallbackResult()
        {
            return _results;
        }
        #endregion


        #region Overrides
        public override string Title
		{
			get
			{
                if (String.IsNullOrEmpty(base.Title))
				    return "Stock Viewer";
                return base.Title;
			}
			set
			{
				base.Title = value;
			}
		}
        public override string Subtitle
        {
            get
            {
                if (String.IsNullOrEmpty(base.Subtitle))
                    return "Zufllige Kurse";
                return base.Subtitle;
            }
        }
		public override WebPartVerbCollection Verbs
		{
			get 
			{
				if (_verbs == null)
				{
					ArrayList a = new ArrayList();
					WebPartVerb verb = new WebPartVerb("Refresh", new WebPartEventHandler(RefreshQuotesFromMenu));
					verb.Text = "Aktualisieren";
					a.Add(verb);
					_verbs = new WebPartVerbCollection(base.Verbs, a);
				}
				return _verbs;
			}
		}
		#endregion


		#region Properties
		// ************************************************************************
        // Gets the collection of stocks to monitor (comma-sep strings)
        [Personalizable]
        [WebBrowsable]
        public string Stocks
        {
			get { return _stocks; }
			set { _stocks = value; }
        }

		// ************************************************************************
		// Gets and sets the refresh rate (in seconds) for the stock list
		[Personalizable]
		[WebBrowsable]
		public int RefreshRate 
		{
			get { return _refreshRate; }
			set { _refreshRate = value; }
		}
		#endregion


		// ************************************************************************
		// Render out the Web part
		protected override void RenderContents (HtmlTextWriter writer)
        {
			if ((_stocks == null) || (_stocks.Length == 0))
			{
				writer.Write("<b>Keine Kurse verfgbar.</b><hr>" + 
					         "<i style=\"font-size:85%\">Geben Sie einige Namen an, und erstellen Sie die Liste der zu berwachenden Aktien. "+
							 "Zum Beispiel \"XYZ, FOO, BAR\"</i>");
			}
			else
			{
				CreateTableOfStocks(writer);
				string msg = "Keine automatische Aktualisierung";
				if (RefreshRate >0)
					msg = String.Format("{0} Sekunden Verzgerung", RefreshRate);
                string text = String.Format(
                    "<br><div style=\"margin:5\"><span style=\"font-size:80%\">" +
					"<span id=\"LastUpdated\">{0}</span> - Kurse zur Verfgung gestellt von <b>RandomQuotes Inc</b>, {1}.</span></div><br>",
                    DateTime.Now.ToString("hh:mm:ss"), 
					msg);
                writer.Write(text);
			}
		}
 
		#region Helper Methods
		// ************************************************************************
		// Build a table with current stock values
		private void CreateTableOfStocks(HtmlTextWriter writer)
		{
			Table t = new Table();
			t.CopyBaseAttributes(this);
            t.Width = Unit.Percentage(100);
			t.CellPadding = 5;
			t.CellSpacing = 0;
			t.GridLines = GridLines.Horizontal;

			// Header
			TableRow head = new TableRow();
			head.TableSection = TableRowSection.TableHeader;
			head.BackColor = Color.SkyBlue;
			head.BorderColor = Color.SkyBlue;
			head.BorderStyle = BorderStyle.Outset;
			head.BorderWidth = Unit.Pixel(2);
			head.Font.Bold = true;

			t.Rows.Add(head);
			TableCell c1 = new TableCell();
            c1.HorizontalAlign = HorizontalAlign.Left;
			c1.Text = "Aktie";
			head.Cells.Add(c1);
			TableCell sep = new TableCell();
			sep.Width = Unit.Pixel(10);
			sep.Text = "&nbsp;";
			head.Cells.Add(sep);
			TableCell c2 = new TableCell();
            c2.HorizontalAlign = HorizontalAlign.Right;
            c2.Text = "Wert";
			head.Cells.Add(c2);

			// Rows
			string[] list = _stocks.Split(new char[] { ',' });
			for (int i = 0; i < list.Length; i++)
			{
				string stockID = list[i];

				TableRow row = new TableRow();
				row.TableSection = TableRowSection.TableBody;
				c1 = new TableCell();
				c1.Text = stockID;
				row.Cells.Add(c1);

				sep = new TableCell();
				sep.Width = Unit.Pixel(10);
				sep.Text = "&nbsp;";
				row.Cells.Add(sep);

				c2 = new TableCell();
               c2.ID = "StockValue" + i.ToString();
				c2.HorizontalAlign = HorizontalAlign.Right;
				c2.Text = GetCurrentValue(stockID);
				row.Cells.Add(c2);
				t.Rows.Add(row);
			}

           // Add some script 
			if (RefreshRate > 0)
			{
				if (!Page.ClientScript.IsStartupScriptRegistered(this.GetType(), "RefreshQuotes"))
				{
					string js2 = GetRefreshScript();
					Page.ClientScript.RegisterStartupScript(this.GetType(), "RefreshQuotes", js2, true);
				}
			}

           // Render out
			t.RenderControl(writer);
		}

		private string GetCurrentValue(string stockID)
		{
			if (_rnd == null)
				_rnd = new Random();

			double d = _rnd.NextDouble();
			int mult = _rnd.Next(1, 100);
			d = d * mult;
			return d.ToString("$##0.0000");
		}

        private string GetRefreshScript()
        {
            StringBuilder sb = new StringBuilder("\r\nfunction RefreshQuotes() {\r\n");
            sb.AppendLine(Page.ClientScript.GetCallbackEventReference(this, "null", "UpdateQuotes", "null"));
            sb.AppendFormat("  window.setTimeout(\"RefreshQuotes()\", {0});", RefreshRate*1000);
			 sb.AppendLine("\r\n}");
            sb.AppendLine(GetUpdateQuotesScript());
            sb.AppendLine("RefreshQuotes();");

            return sb.ToString();
        }

        private string GetUpdateQuotesScript()
        {
            StringBuilder sb = new StringBuilder("function UpdateQuotes(response, context) {\r\n");
            //sb.AppendLine("  alert(response); ");
            sb.AppendLine("  var quotes = response.split(',');");
            sb.AppendLine("  for(i=1; i<quotes.length; i++)");
            sb.AppendLine("  {");
            sb.AppendLine("    var elem = document.getElementById(\"StockValue\" + (i-1).toString());");
            sb.AppendLine("    elem.innerHTML = quotes[i];");  
            sb.AppendLine("  }");
			 sb.AppendLine("  var now = document.getElementById(\"LastUpdated\");");
			 sb.AppendLine("  now.innerHTML = quotes[0];");  
			 sb.AppendLine("}");
            return sb.ToString();
        }

		private void RefreshQuotesFromMenu(object sender, WebPartEventArgs e)
		{
		}
		#endregion
	}
}

