using System;
using System.Data;
using System.Configuration;
using System.Collections.Specialized;
using System.Web;
using System.Drawing;
using System.Web.UI;
using System.Web.UI.WebControls;



public partial class UserLinks : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            InitProfile();
            InitOptionsDialog();
        }

        ApplyPagePersonalization();
    }

    private void InitProfile()
    {
        // Set a default value for the Links profile property. 
        // Note that reference types can't have a default value specified in web.config
        // Reference objects are NEVER null.
        if (String.IsNullOrEmpty(Profile.BackColor))
            Profile.BackColor = InfoPanel.BackColor.Name;
        if (String.IsNullOrEmpty(Profile.ForeColor))
            Profile.ForeColor = InfoPanel.ForeColor.Name;
    }

    private void InitOptionsDialog()
    {
        NewBackColor.Text = Profile.BackColor;
        NewForeColor.Text = Profile.ForeColor;
    }

    private void ApplyPagePersonalization()
    {
        // Handle the color properties
        InfoPanel.ForeColor = ColorTranslator.FromHtml(Profile.ForeColor);
        InfoPanel.BackColor = ColorTranslator.FromHtml(Profile.BackColor);


        // Handle the Links property
        Favorites.Controls.Clear();
        if(Profile.Links.Count == 0)
            Favorites.Controls.Add (new LiteralControl("No links available."));
        else
            foreach (object o in Profile.Links)
            {
                HyperLink h = new HyperLink ();

                h.Text = o.ToString ();
                h.NavigateUrl = o.ToString ();
                Favorites.Controls.Add(h);
                Favorites.Controls.Add(new LiteralControl("<br />"));
            }
    }

    protected void OnEditOptions(object sender, EventArgs e)
    {
        Options.ActiveViewIndex = 1;
    }

    protected void OnCloseOptions(object sender, EventArgs e)
    {
        Options.ActiveViewIndex = 0;
    }
    
    protected void OnSetBackColor(object sender, EventArgs e)
    {
        Profile.BackColor = NewBackColor.Text;
        ApplyPagePersonalization();
    }

    protected void OnSetForeColor(object sender, EventArgs e)
    {
        Profile.ForeColor = NewForeColor.Text;
        ApplyPagePersonalization();
    }

    protected void OnAddLink(object sender, EventArgs e)
    {
        Profile.Links.Add(NewLink.Text);
        ApplyPagePersonalization();
    }
}
