using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Threading;
using System.Xml;
using System.Net;
using System.IO;
using System.Text;
using System.Xml.XPath;


public partial class MultiTaskResultAsync : System.Web.UI.Page
{
    private const string RSSFEED1 = "http://weblogs.asp.net/despos/rss.aspx";
    private const string RSSFEED2 = "http://blogs.ugidotnet.org/dinoes/rss.aspx";
    private WebRequest req1, req2;
    IAsyncResult ar1, ar2;
    public string rssData;


    void Page_Load (object sender, EventArgs e)
    {
        AddOnPreRenderCompleteAsync (
            new BeginEventHandler(BeginTask),
            new EndEventHandler(EndTask));
    }


    IAsyncResult BeginTask(object sender, EventArgs e, AsyncCallback cb, object state)
    {
        // Create the custom AsyncResult object
        CompositeAsyncResult ar = new CompositeAsyncResult(cb, state, 2);
        
        // Fire the first request
        req1 = WebRequest.Create(RSSFEED1);
        ar1 = req1.BeginGetResponse(ar.Callback, state);

        // Fire the second request
        req2 = WebRequest.Create(RSSFEED2);
        ar2 = req2.BeginGetResponse(ar.Callback, state);

        return ar;
    }


    void EndTask(IAsyncResult ar)
    {
        string text;
        WebResponse response;
        StreamReader reader;

        using (response = req1.EndGetResponse(ar1))
        {
            using (reader = new StreamReader(response.GetResponseStream()))
            {
                text = reader.ReadToEnd();
            }

            // Process the RSS data
            rssData += ProcessFeed(text);
        }

        using (response = req2.EndGetResponse(ar2))
        {
            using (reader = new StreamReader(response.GetResponseStream()))
            {
                text = reader.ReadToEnd();
            }

            // Process the RSS data
            rssData += ProcessFeed(text);
        }

    }
    
    string ProcessFeed(string feed)
    {
        StringBuilder sb = new StringBuilder();

        XmlDocument doc = new XmlDocument();
        doc.LoadXml(feed);

        XPathNavigator nav = doc.CreateNavigator();
        XPathNodeIterator iterator;
        
        // Title and description
        iterator = nav.Select("/rss/channel/title");
        iterator.MoveNext();
        sb.AppendFormat("<h3>{0}</h3>", iterator.Current.Value);
        iterator = nav.Select("/rss/channel/description");
        iterator.MoveNext();
        sb.AppendFormat("<i>{0}</i>", iterator.Current.Value);

        XmlNodeList items = doc.SelectNodes("/rss/channel/item");
        sb.Append("<ul>");
        foreach(XmlNode n in items)
        {
            XmlNode title = n.SelectSingleNode("title");
            XmlNode link = n.SelectSingleNode("link");
            XmlNode date = n.SelectSingleNode("pubDate");
            sb.AppendFormat("<li><a href=\"{0}\">{1}</a> on {2}</li>", link.InnerText, title.InnerText, date.InnerText);
        }
        sb.Append("</ul>");

        return sb.ToString();
    }
}


