using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Threading;
using System.Xml;
using System.Net;
using System.IO;
using System.Text;
using System.Xml.XPath;


public partial class SimpleRssAsync : System.Web.UI.Page
{
    private const string RSSFEED1 = "http://weblogs.asp.net/despos/rss.aspx";
    private const string RSSFEED2 = "http://blogs.ugidotnet.org/dinoes/rss.aspx";
    private WebRequest req;
    public string rssData;


    void Page_Load (object sender, EventArgs e)
    {
        AddOnPreRenderCompleteAsync(
            new BeginEventHandler(BeginTask),
            new EndEventHandler(EndTask),
            RSSFEED1);

        PageAsyncTask task1 = new PageAsyncTask(
            new BeginEventHandler(BeginTask),
            new EndEventHandler(EndTask),
            null,
            RSSFEED1);
        PageAsyncTask task2 = new PageAsyncTask(
            new BeginEventHandler(BeginTask),
            new EndEventHandler(EndTask),
            null,
            RSSFEED2);

        RegisterAsyncTask(task1);
        RegisterAsyncTask(task2);
    }
    
    IAsyncResult BeginTask(object sender, EventArgs e, AsyncCallback cb, object state)
    {
        // Trace 
        Utils.TraceThread("BeginTask, Anfang");

        // Prepare to make a Web request
        string blog = (string)state;
        req = WebRequest.Create(blog);

        // Begins the operation and return an object that implements IAsyncResult
        IAsyncResult obj = req.BeginGetResponse(cb, state);
        Utils.TraceThread("Blog lesen");

        // Trace
        Utils.TraceThread("BeginTask, Ende");
        Utils.TraceThread();
        return obj;
    }

    void EndTask(IAsyncResult ar)
    {
        // This code will be called on a completion-pool thread 

        // Trace 
		string displayThread = "Thread=" + Thread.CurrentThread.ManagedThreadId.ToString();
		Trace.Warn(displayThread, "EndTask, Anfang");


        string text;
        using (WebResponse response = req.EndGetResponse(ar))
        {
			Trace.Warn(displayThread, "Operation abgeschlossen");
            using (StreamReader reader = new StreamReader(response.GetResponseStream()))
            {
                text = reader.ReadToEnd();
            }

            // Process the RSS data
            rssData += ProcessFeed(text);
        }

        // Trace
		Trace.Warn(displayThread, "EndTask, Ende");

        
        // The page output is updated using an ASP-style code block 
        // in the ASPX source that displays the contents of the rssData variable
    }


    string ProcessFeed(string feed)
    {
        StringBuilder sb = new StringBuilder();

        XmlDocument doc = new XmlDocument();
        doc.LoadXml(feed);

        XPathNavigator nav = doc.CreateNavigator();
        XPathNodeIterator iterator;
        
        // Title and description
        iterator = nav.Select("/rss/channel/title");
        iterator.MoveNext();
        sb.AppendFormat("<h3>{0}</h3>", iterator.Current.Value);
        iterator = nav.Select("/rss/channel/description");
        iterator.MoveNext();
        sb.AppendFormat("<i>{0}</i>", iterator.Current.Value);

        XmlNodeList items = doc.SelectNodes("/rss/channel/item");
        sb.Append("<ul>");
        foreach(XmlNode n in items)
        {
            XmlNode title = n.SelectSingleNode("title");
            XmlNode link = n.SelectSingleNode("link");
            XmlNode date = n.SelectSingleNode("pubDate");
            sb.AppendFormat("<li><a href=\"{0}\">{1}</a> on {2}</li>", link.InnerText, title.InnerText, date.InnerText);
        }
        sb.Append("</ul>");

        return sb.ToString();
    }
}


