using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;



public partial class ShowInfo : System.Web.UI.Page, ICallbackEventHandler 
{
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            // Populate the drop-down list
            cboEmployees.DataSource = GetListOfNames();
            cboEmployees.DataBind();

            // Prepare the Javascript function to call
            string rpc = ClientScript.GetCallbackEventReference(
                this,
                "document.forms[0].elements['cboEmployees'].value",
                "UpdateEmployeeViewHandler",
                "null",
                "null",
                false);

            // Bind it to a button
            string js = String.Format("javascript:{0}", rpc);
            buttonTrigger.Attributes["onclick"] = js;
        }
    }

    #region ICallbackEventHandler
    private string _results;
    string ICallbackEventHandler.GetCallbackResult()
    {
        return _results;
    }

	void ICallbackEventHandler.RaiseCallbackEvent(string eventArgument)
	{
		// Get more info about the specified employee
        int empID = -1;
        bool success = Int32.TryParse(eventArgument, out empID);
		EmployeeInfo emp = GetEmployeeDetails(empID);

		string[] buf = new string[6];
		buf[0] = emp.ID.ToString();
		buf[1] = emp.FirstName;
		buf[2] = emp.LastName;
		buf[3] = emp.Title;
		buf[4] = emp.Country;
		buf[5] = emp.Notes;
		_results =  String.Join(",", buf);
	}
    #endregion

    #region Helpers
    public class EmployeeInfo
    {
        public int ID;
        public string FirstName;
        public string LastName;
        public string Title;
        public string Country;
        public string Notes;
    }

    private DataTable GetListOfNames()
    {
        // Get the lastnames of all employees
        SqlDataAdapter adapter = new SqlDataAdapter(
            "SELECT employeeid, lastname FROM employees",
            ConfigurationManager.ConnectionStrings["LocalNWind"].ConnectionString);
        DataTable table = new DataTable();
        adapter.Fill(table);
        return table;
    }

    public EmployeeInfo GetEmployeeDetails(int empID)
    {
        // Get details about the specified employee
        SqlDataAdapter adapter = new SqlDataAdapter(
            "SELECT firstname, lastname, title, country, notes FROM employees WHERE employeeid=@id",
            ConfigurationManager.ConnectionStrings["LocalNWind"].ConnectionString);
        adapter.SelectCommand.Parameters.AddWithValue("@id", empID);

        DataTable table = new DataTable();
        adapter.Fill(table);

        // Execute the command and populate the return buffer
        DataRow row = table.Rows[0];
        EmployeeInfo info = new EmployeeInfo();
        info.ID = empID;
        info.FirstName = row["firstname"].ToString();
        info.LastName = row["lastname"].ToString();
        info.Title = row["title"].ToString();
        info.Country = row["country"].ToString();
        info.Notes = row["notes"].ToString();

        return info;
    }
#endregion

}
