using System;
using System.Data;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace ProAspNet20.Advanced.CS.Components
{
    #region CLASS ButtonClickedEventArgs
    public class ButtonClickedEventArgs : EventArgs
    {
        public string ButtonCommand;
        public int ButtonIndex;
    }
    #endregion

    #region CLASS RefreshDataEventArgs
    public class RefreshDataEventArgs : EventArgs
    {
        public object DataSource; 
    }
    #endregion


    public partial class ButtonList : System.Web.UI.UserControl
    {
        #region Internal Handlers
        protected void Page_Init(object sender, EventArgs e)
        {
            HookUpEvents();
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
                BindData();
        }

        protected void ItemCommand(object sender, RepeaterCommandEventArgs e)
        {
            CurrentButtonIndex = e.Item.ItemIndex;


            // Fire the event
            ButtonClickedEventArgs ev = new ButtonClickedEventArgs();
            ev.ButtonCommand = e.CommandName;
            ev.ButtonIndex = CurrentButtonIndex;
            OnButtonClicked(ev);

            BindData();
        }

        protected void ItemCreated(object sender, RepeaterItemEventArgs e)
        {
            if (e.Item.ItemType != ListItemType.Item &&
                e.Item.ItemType != ListItemType.AlternatingItem)
                return;

            Button btn = (Button)e.Item.FindControl("TheButton");
            if (e.Item.ItemIndex == CurrentButtonIndex)
            {
                btn.BackColor = SelectedBackColor;
                btn.ForeColor = SelectedForeColor;
                btn.Height = Unit.Pixel(23);
                btn.Width = Unit.Pixel(ButtonWidth * 150 / 100);
            }
            else
            {
                btn.BackColor = BackColor;
                btn.ForeColor = ForeColor;
                btn.Height = Unit.Pixel(20);
                btn.Width = Unit.Pixel(ButtonWidth);
            }
        }

        protected void ItemDataBound(object sender, RepeaterItemEventArgs e)
        {
            if (e.Item.ItemType != ListItemType.Item &&
                e.Item.ItemType != ListItemType.AlternatingItem)
                return;

            RepeaterItem item = e.Item;
            Button btn = (Button)item.FindControl("TheButton");
            btn.Text = DataBinder.GetPropertyValue(item.DataItem, DataTextField).ToString();
            btn.CommandName = DataBinder.GetPropertyValue(item.DataItem, DataValueField).ToString();
        }

        #endregion


        #region Events
        public event EventHandler<ButtonClickedEventArgs> ButtonClicked;
        public event EventHandler<RefreshDataEventArgs> RefreshData;

        protected virtual void OnButtonClicked(ButtonClickedEventArgs e)
        {
            if (ButtonClicked != null)
                ButtonClicked(this, e);
        }

        protected virtual void OnRefreshData(RefreshDataEventArgs e)
        {
            if (RefreshData != null)
                RefreshData(this, e);
        }       
        #endregion


        #region Properties
        public object DataSource
        {
            get 
            {
                object o = Cache[ID];
                if (o == null)
                {
                    RefreshDataEventArgs ev = new RefreshDataEventArgs();
                    OnRefreshData(ev);
                    o = ev.DataSource;
                }
                return o; 
            }
            set { Cache[ID] = value; }
        }

        public string DataTextField
        {
            get 
            {
                object o = ViewState["DataTextField"];
                if (o == null)
                    return String.Empty;
                return (string) o; 
            }
            set { ViewState["DataTextField"] = value; }
        }

        public string DataValueField
        {
            get
            {
                object o = ViewState["DataValueField"];
                if (o == null)
                    return String.Empty;
                return (string)o;
            }
            set { ViewState["DataValueField"] = value; }
        }

        public int CurrentButtonIndex
        {
            get
            {
                object o = ViewState["CurrentButtonIndex"];
                if (o == null)
                    return 0;
                return (int) o;
            }
            set
            {
                ViewState["CurrentButtonIndex"] = value;
                if (__theSep != null)
                    __theSep.BackColor = (value == -1 ? BackColor : SelectedBackColor);
            }
        }

        public int ButtonWidth
        {
            get
            {
                object o = ViewState["ButtonWidth"];
                if (o == null)
                    return 25;
                return (int)o;
            }
            set { ViewState["ButtonWidth"] = value; }
        }

        #endregion


        #region Other Non-Public Properties
        protected Color BackColor = Color.FromName("#eeeeee");
        protected Color ForeColor = Color.Black;
        protected Color BorderColor = Color.Gray;
        protected Color SelectedBackColor = Color.Gray;
        protected Color SelectedForeColor = Color.White;
        protected Color SelectedBorderColor = Color.Gray;
        #endregion


        #region Helpers
        private void BindData()
        {
            __theMenu.DataSource = DataSource;
            __theMenu.DataBind();
        }
        
        private void HookUpEvents()
        {
            __theMenu.ItemCommand += new RepeaterCommandEventHandler(ItemCommand);
            __theMenu.ItemCreated += new RepeaterItemEventHandler(ItemCreated);
            __theMenu.ItemDataBound += new RepeaterItemEventHandler(ItemDataBound);
        }
        #endregion
    }


}