using System;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace ProAspNet20.Advanced.CS.Components
{
    public partial class Tabstrip : System.Web.UI.UserControl
    {
        #region Properties
        // Tabs to display
        public ArrayList Tabs = new ArrayList();

        // Current Tab
        public int CurrentTabIndex
        {
            get
            {
                object o = ViewState["CurrentTabIndex"];
                if (o == null)
                    return 0;
                return (int)o;
            }
        }

        // Background color
        public Color BackColor
        {
            get
            {
                object o = ViewState["BackColor"];
                if (o == null)
                    return Color.Silver;
                return (Color)o;
            }
            set { ViewState["BackColor"] = value; }
        }

        // Selected background color
        public Color SelectedBackColor
        {
            get
            {
                object o = ViewState["SelectedBackColor"];
                if (o == null)
                    return Color.Orange;
                return (Color)o;
            }
            set { ViewState["SelectedBackColor"] = value; }
        }

        // Foreground color
        public Color ForeColor
        {
            get
            {
                object o = ViewState["ForeColor"];
                if (o == null)
                    return Color.White;
                return (Color)o;
            }
            set { ViewState["ForeColor"] = value; }
        }

        // Selected foreground color
        public Color SelectedForeColor
        {
            get
            {
                object o = ViewState["SelectedForeColor"];
                if (o == null)
                    return Color.White;
                return (Color)o;
            }
            set { ViewState["SelectedForeColor"] = value; }
        }
        #endregion


        #region Events
        public event EventHandler<SelectionChangedEventArgs> SelectionChanged;
        #endregion


        #region Page Object Model
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
                BindData();
        }

        protected void ItemCommand(object sender, RepeaterCommandEventArgs e)
        {
            Select(e.Item.ItemIndex);
        }

        public void Select(int index)
        {
            // Ensure the index is a valid value
            if (index < 0 || index > Tabs.Count)
                index = 0;

            // Updates the current index. Must write to the view state
            // because the CurrentTabIndex property is read-only
            ViewState["CurrentTabIndex"] = index;

            // Ensure the bottom panel is of the selected color
            __theSep.BackColor = SelectedBackColor;

            // Refresh the UI
            BindData();

            // Fire the event to the client
            SelectionChangedEventArgs ev = new SelectionChangedEventArgs();
            ev.Position = CurrentTabIndex;
            OnSelectionChanged(ev);
        }
        #endregion


        #region Private Members
        private void BindData()
        {
            __theTabStrip.DataSource = Tabs;
            __theTabStrip.DataBind();
        }

        protected Color SetBackColor(object elem)
        {
            RepeaterItem item = (RepeaterItem)elem;
            if (item.ItemIndex == CurrentTabIndex)
                return SelectedBackColor;
            return BackColor;
        }

        protected Color SetForeColor(object elem)
        {
            RepeaterItem item = (RepeaterItem)elem;
            if (item.ItemIndex == CurrentTabIndex)
                return SelectedForeColor;
            return ForeColor;
        }

        protected Color SetBorderColor(object elem)
        {
            RepeaterItem item = (RepeaterItem)elem;
            if (item.ItemIndex == CurrentTabIndex)
                return SelectedBackColor;
            return Color.Black;
        }

        // Helper function that fires the event by executing user-defined code
        protected virtual void OnSelectionChanged(SelectionChangedEventArgs e)
        {
            // SelectionChanged is the event property
            if (SelectionChanged != null)
                SelectionChanged(this, e);
        }
        #endregion
    }


    #region Class SelectionChangedEventArgs
    public class SelectionChangedEventArgs : EventArgs
    {
        public int Position;
    }
    #endregion

}